""" Utility function for vizex """

from math import ceil
from colored import fg, attr, stylize

def bytes_to_human_readable(bytes: int, suffix='B') -> str:
    """
    Converts bytes into the appropriate human
    readable unit with a relevant suffix.
    """
    for unit in ['','K','M','G','T','P','E','Z']:
        if abs(bytes) < 1024.0:
            return f'{bytes:3.1f} {unit}{suffix}'
        bytes /= 1024.0
    return f'{bytes:.1f} {"Y"}{suffix}'

def ints_to_human_readable(disk: dict) -> dict:
    """
    Converts the dictionary of integers
    into the human readable strings.
    """
    result = {}
    try:
        for key in disk:
            result[key] = bytes_to_human_readable(disk[key])
    except:
        result[key] = disk[key]
    return result

def printml(folder: list, cols: int = 1) -> None:
    """Prints multiline strings side by side."""
    size = len(folder)
    incr = ceil(size / cols)
    end, start = 0, 0
    while True:
        if end >= size:
            break
        end += incr
        # Check if the end index exceeds the last index
        if end > size:
            end = size
        lines = [folder[i].splitlines() for i in range(start, end)]
        for line in zip(*lines):
            print(*line, sep='  ')
        print()
        start = end


import psutil

from math import ceil
from charts import Chart, HorizontalBarChart, Options
from colored import fg, attr, stylize
from tools import bytes_to_human_readable, ints_to_human_readable


class DiskUsage:
    """
    Personalize and visualize the disk space usage in the terminal

    options:
        barh: visualize values as a horizontal bar
        barv: visualiz disk space as a vertical bar
        pie: visualize as a pie charts
    """

    def __init__(self,
                path: str = "",
                exclude: list=None,
                details: bool=False,
                every: bool=False) -> None:
        self.path = path
        if exclude is None:
            self.exclude = []
        else:
            self.exclude = exclude
        self.details = details
        self.every = every

    def print_charts(self, options: Options=None) -> None:
        """Prints the charts based on user selection type"""
        if options is None:
            options = Options()

        if self.path:
            parts = self.grab_specific(self.path[0])
        else:
            parts = self.grab_partitions(exclude=self.exclude,
                                        every=self.every)

        chrt = HorizontalBarChart(options)
        for partname in parts:
            self.print_disk_chart(chrt, partname, parts[partname])

    def print_disk_chart(
            self, chart: Chart, partname: str, part: dict
        ) -> None:
        ch = chart
        title = (partname,)
        pre_graph_text = self.create_stats(part)
        if self.details:
            footer = self.create_details_text(part)
        else:
            footer = None

        maximum = part["total"]
        current = part["used"]
        post_graph_text = self.create_pct_used(part['percent'])

        ch.chart(
            post_graph_text=post_graph_text,
            title=title[0],
            pre_graph_text=pre_graph_text,
            footer=footer,
            maximum=maximum,
            current=current,
        )
        print()

    def print_barchart(self, disk_name: str, disk: dict) -> None:
        """
        Prints disk usage in the Terminal with horizontal bars
        """
        print(f"{stylize(disk_name, self.header + self.style)}")
        print(self.create_stats(disk))
        chart = ChartPrint(self.graph, self.symbol)
        print(
            "",
            chart.draw_horizontal_bar(capacity=disk["total"],
                                used=disk["used"]),
                                self.create_warning(disk["percent"]),
        )
        if self.details:
            print(self.details_text(disk))
            print()

    def grab_root(self) -> dict:
        """Grab the data about the root partition"""
        root = {
            "total": psutil.disk_usage("/").total,
            "used": psutil.disk_usage("/").used,
            "free": psutil.disk_usage("/").free,
            "percent": psutil.disk_usage("/").percent,
            "fstype": psutil.disk_partitions(all=False)[0][2],
            "mountpoint": "/",
        }
        return root

    def grab_partitions(self, exclude: list, every: bool) -> dict:
        """Grabs all the partitions from the user's PC."""
        if self.exclude is None:
            exclude = []
        disks = {}
        if not every:
            disks['root'] = self.grab_root()
        disk_parts = psutil.disk_partitions(all=every)
        for disk in disk_parts[1:]:
            # Exclude mounpoints created by snap
            if disk.device.startswith("/dev/loop"):
                continue
            # Check that part name is not in the excluded list
            if disk[1].split("/")[-1] in exclude:
                continue
            try:
                if psutil.disk_usage(disk[1]).total > 0:
                    disks[disk[1].split("/")[-1]] = {
                        "total": psutil.disk_usage(disk[1]).total,
                        "used": psutil.disk_usage(disk[1]).used,
                        "free": psutil.disk_usage(disk[1]).free,
                        "percent": psutil.disk_usage(disk[1]).percent,
                        "fstype": disk.fstype,
                        "mountpoint": disk.mountpoint,
                    }
            except:
                continue
        return disks

    def grab_specific(self, disk_path: str) -> dict:
        """
        Grabs data for the partition of the user specified path
        """
        disks = {}
        disks[disk_path] = {
            "total": psutil.disk_usage(disk_path).total,
            "used": psutil.disk_usage(disk_path).used,
            "free": psutil.disk_usage(disk_path).free,
            "percent": psutil.disk_usage(disk_path).percent,
            "fstype": "N/A",
            "mountpoint": "N/A",
        }
        return disks

    def create_details_text(self, disk: dict) -> str:
        return f"fstype={disk['fstype']}\tmountpoint={disk['mountpoint']}"

    def create_stats(self, disk: dict) -> str:
        r = ints_to_human_readable(disk)
        return f"Total: {r['total']}\t Used: {r['used']}\t Free: {r['free']}"

    def create_pct_used(self, usage) -> str:
        """Create disk usage percent with warning color"""
        use = str(usage) + '% full'
        if usage >= 80:
            return f"{stylize(use, attr('blink') + fg(9))}"                  
        elif usage >= 60:
            return f"{stylize(use, fg(214))}"
        else:
            return f"{stylize(use, fg(82))}"


if __name__ == "__main__":
    self = DiskUsage()
    parts = self.grab_partitions(exclude=[], every=False)

    for partname in parts:
        part = parts[partname]
        ch = HorizontalBarChart()
        title = (partname,)
        pre_graph_text = self.create_stats(part)
        footer = self.create_details_text(part)
        maximum = part["total"]
        current = part["used"]
        post_graph_text = self.create_pct_used(part['percent'])

        ch.chart(
            post_graph_text=post_graph_text,
            title=title[0],
            pre_graph_text=pre_graph_text,
            footer=footer,
            maximum=maximum,
            current=current,
        )
        print()



""" Text Chart Printing Functions """

from math import ceil
from colored import fg, attr, stylize

class Options:
    """Options"""

    _graph_color = fg("white")
    _header_color = fg("red")
    _header_style = attr('bold')
    _text_color = fg("white")
    _fsymbol = "█"
    _msymbol = "▒"
    _esymbol = "░"

    def __init__(self):
        return

    @property
    def graph_color(self):
        """ Graph Color """
        return self._graph_color

    @graph_color.setter
    def graph_color(self, color: str):
        self._graph_color = fg(color)

    @property
    def header_color(self):
        """ Header Color """
        return self._header_color

    @header_color.setter
    def header_color(self, color: str):
        self._header_color = fg(color)

    @property
    def header_style(self):
        """ header style """
        return self._header_style
    
    @header_style.setter
    def header_style(self, style: str):
        self._header_style = attr(style)

    @property
    def text_color(self):
        """ text color """
        return self._text_color

    @text_color.setter
    def text_color(self, color: str):
        self._text_color = fg(color)

    @property
    def symbol(self):
        """ graph symbols to use """
        return (self._fsymbol, self._msymbol, self.esymbol)

    @symbol.setter
    def symbol(self, symbol: str):
        if symbol:
            self._fsymbol = symbol
            self._msymbol = ">"
            self._esymbol = "-"
        else:
            self._fsymbol = "█"
            self._msymbol = "▒"
            self._esymbol = "░"

    @property
    def fsymbol(self):
        """ The full symbol """
        return self._fsymbol

    @property
    def esymbol(self):
        """ The empty symbol """
        return self._esymbol

    @property
    def msymbol(self):
        """ The middle symbol """
        return self._msymbol


class Chart:
    """Abstract base object for charts"""

    def __init__(self, options: Options = None):
        if options is None:
            self.options = Options()
        else:
            self.options = options


class HorizontalBarChart(Chart):
    """
    Draws horizontal chart with user selected color and symbol
    """

    def chart(self,
            title: str,
            pre_graph_text: str,
            post_graph_text: str,
            footer: str,
            maximum: float,
            current: float) -> None:
        print(
            stylize(title, self.options.header_color + self.options.header_style)
        )

        if pre_graph_text:
            print(stylize(pre_graph_text, self.options.text_color))

        print(
            "%s"
            % stylize(
                self.draw_horizontal_bar(maximum, current),
                self.options.graph_color
            ),
            end=" ",
        )
        if post_graph_text:
            print(stylize(post_graph_text, self.options.text_color))
        else:
            print()

        if footer:
            print(stylize(footer, self.options.text_color))

    def draw_horizontal_bar(self, maximum: int, current: int) -> str:
        """Draw a horizontal bar chart"""
        textBar = ""
        usage = int((current / maximum) * 38)
        for i in range(1, usage + 1):
            textBar += self.options.fsymbol
        textBar += self.options.msymbol
        for i in range(1, 39 - usage):
            textBar += self.options.esymbol
        # check if the user set up graph color
        if "█" not in self.options.fsymbol:
            return f"[{textBar}]"
        return textBar


class VerticalBarChart(Chart):

    def draw_vertical_bar(self, capacity: int, used: int) -> str:
        """Draw a vertical bar chart"""
        textBar = "\n"
        n = (used / capacity) * 8
        # If the usage is below 1% print empty chart
        if n < 0.1:
            n = 0
        else:
            n = ceil(n)
        textBar += f"{self.esymbol * 9}  \n" * (8 - n)
        textBar += f"{self.fsymbol * 9}  \n" * n
        return textBar


if __name__ == "__main__":
    ch = BarChart()
    ch.options.post_graph_color = "green"
    ch.options.pre_graph_color = "blue"
    ch.options.footer_color = "yellow"
    ch.chart(
        title="Test Content",
        maximum=100,
        current=32,
        pre_graph_text="Lorem: Sit ea dolore ad accusantium",
        post_graph_text="Good job!",
        footer="This concludes our test",
    )



"""cli.py - Command line interface for vizex"""

import click

from disks import DiskUsage
from charts import Options
from colored import fg, attr, stylize

# Command line arguments and options for cli
@click.command()
@click.argument(
    "chart",
    nargs=1,
    default="barh",
    metavar="[CHART_TYPE]"
)
@click.option(
    "-P",
    "--path",
    default=None,
    multiple=True,
    help="Print only specific partition/disk",
)
@click.option(
    "-X",
    "--exclude",
    default=None,
    multiple=True,
    help="Select partition you want to exclude",
)
@click.option(
    "--every",
    is_flag=True,
    help="Display information for all the disks"
)
@click.option(
    "--details",
    is_flag=True,
    help="Display additinal details like fstype and mountpoint",
)
@click.option(
    "-d",
    "--header",
    default=None,
    type=str,
    metavar="[COLOR]",
    help="Set the partition name color",
)
@click.option(
    "-s",
    "--style",
    default=None,
    type=str,
    metavar="[ATTR]",
    help="Change the style of the header's display",
)
@click.option(
    "-t",
    "--text",
    default=None,
    type=str,
    metavar="[COLOR]",
    help="Set the color of the regular text",
)
@click.option(
    "-g",
    "--graph",
    default=None,
    type=str,
    metavar="[COLOR]",
    help="Change the color of the bar graph",
)
@click.option(
    "-m",
    "--mark",
    default=None,
    help="Choose the symbols used for the graph"
)
def cli(chart, path, every, details, exclude,
        header, style, text, graph, mark) -> None:
    """
    ** Displays Disk Usage in the terminal, graphically. **

    Customize visuals by setting colors and attributes.

    Select one of the available graph types:
        barv : Vertical Bars
        *barh : Horizontal Bars (buggy)
        *pie : Pie Chart (coming soon)

    COLORS: light_red, red, dark_red, dark_blue, blue,
        cyan, yellow, green, pink, white, black, purple,
        neon, grey, beige, orange, magenta, peach.

    ATTRIBUTES: bold, dim, underlined, blink, reverse.
    """
    chart = "barh"
    options: Options = Options()
    if mark:
        options.symbol = mark
    if header:
        options.header_color = header
    if text:
        options.text_color = text
    if graph:
        options.graph_color = graph
    if style:
        options.header_style = style

    chart = chart
    exclude_list = list(exclude)

    renderer = None
    if chart == "barh":
        renderer = DiskUsage(
            path=path, exclude=exclude_list, details=details, every=every
        )
    renderer.print_charts(options)


if __name__ == "__main__":
    cli()
